# Gulp File Loader

_Auto generate import-only files for any file type. SCSS, JS, Pug, whatever you want._

[![Supported Node version](https://img.shields.io/node/v/gulp-file-loader.svg?style=for-the-badge)](https://nodejs.org/en/)

Are you sick of having to manually manage files that are purely just a bunch of import statements?

Wouldn't it be awesome if Gulp could just look at your file system and manage this busy work for you?

That is where Gulp File Loader comes in. Gulp File Loader will automatically manage these import-only files for you giving you more time to worry about the important stuff.

Due to it's high level of customization, Gulp File Loader is able to generate any import file you can imagine. SCSS, JS, Pug, PHP, you name it, it can create an import file for it (assuming the language supports import functionality in some way).

Gulp File Loader also has the ability to remember the order that imports are declared in. If you have ever had any experience with glob-loading SCSS files, you will know the pain of trying to get `alpha.scss` to override styles in `beta.scss`. With Gulp File Loader, simply rearrange the import statements and you're done!

> **Note:** if using Gulp File Loader with Sass files, it is best used [*in combination*](#use-in-combination-with-gulp-sass-glob) with [Gulp Sass Glob](https://www.npmjs.com/package/gulp-sass-glob), not as a complete replacement for it.

## Contents <!-- omit in toc -->

- [Before and after Gulp File Loader](#before-and-after-gulp-file-loader)
  - [SCSS Before](#scss-before)
  - [SCSS After](#scss-after)
  - [JS before](#js-before)
  - [JS after](#js-after)
- [Install](#install)
- [Using the SCSS preset](#using-the-scss-preset)
- [Use in *combination* with Gulp Sass Glob](#use-in-combination-with-gulp-sass-glob)
  - [Gulp 3 combination](#gulp-3-combination)
  - [Gulp 4 combination](#gulp-4-combination)
  - [Using a combination inside main.scss](#using-a-combination-inside-mainscss)
- [All available presets](#all-available-presets)
  - [Overriding a preset](#overriding-a-preset)
- [JS configuration examples](#js-configuration-examples)
  - [Rollup](#rollup)
    - [Rollup in Gulp 4](#rollup-in-gulp-4)
    - [Rollup in Gulp 3](#rollup-in-gulp-3)
  - [Browserify](#browserify)
  - [Making use of the generated JS file](#making-use-of-the-generated-js-file)
- [How to do custom configurations](#how-to-do-custom-configurations)
  - [Manual SCSS set up](#manual-scss-set-up)
  - [Manual JS set up](#manual-js-set-up)
- [Understanding the `format` and `template` settings](#understanding-the-format-and-template-settings)
  - [The `$name` placeholder](#the-name-placeholder)
  - [The `$path` placeholder](#the-path-placeholder)
  - [Using indents](#using-indents)
- [The `retainOrder` setting](#the-retainorder-setting)
- [Settings reference guide](#settings-reference-guide)
- [Change Log](#change-log)


## Before and after Gulp File Loader

### SCSS Before

```scss
// main.scss (manually edited)

@import "../components/component-A/A.scss";
@import "../components/component-B/B.scss";
@import "../components/component-C/C.scss";
@import "../components/component-D/D.scss";
```

### SCSS After
```scss
// main.scss (manually edited)

@import "./file-loader.scss";
```

```scss
// file-loader.scss (auto-generated)
// retains the order that imports are declared in if edited manually

@import "../components/component-A/A.scss";
@import "../components/component-B/B.scss";
@import "../components/component-C/C.scss";
@import "../components/component-D/D.scss";
```

### JS before

```js
// main.js (manually edited)

import $ from 'jquery';

import A from "../components/component-A/A.js";
import B from "../components/component-B/B.js";
import C from "../components/component-C/C.js";
import D from "../components/component-D/D.js";

$(() => {
  A();
  B();
  C();
  D();
})
```

### JS after

```js
// main.js (manually edited)

import $ from 'jquery';

import fileLoader from "./file-loader.js";

$(() => {
  fileLoader();
})
```

```js
// file-loader.js (auto-generated)

import A from "../components/component-A/A.js";
import B from "../components/component-B/B.js";
import C from "../components/component-C/C.js";
import D from "../components/component-D/D.js";

export default function() {
  A();
  B();
  C();
  D();
})
```

## Install

Install Gulp File Loader using the following command:

```
npm install gulp-file-loader --save-dev
```

For my examples, I am assuming that the project folder structure looks like this:

```
[root]
|  source
|  |  components
|  |  |  [component-folders]
|  |  |  |  [component-name].js
|  |  |  |  [component-name].scss
|  |  js
|  |  |  file-loader.js
|  |  |  main.js
|  |  scss
|  |  |  config
|  |  |  |  [scss-config-files]
|  |  |  file-loader.scss
|  |  |  main.scss
|  build
|  |  assets
|  |  |  css
|  |  |  |  main.css
|  |  |  js
|  |  |  |  main.js
| gulpfile.js
```

## Using the SCSS preset

Writing out all of the required settings manually for this plugin can be a bit tiresome. As of version 2.0.0 I've included some common preset settings that you can use as defaults. All you need to do for a basic SCSS set up is the following. You will need the `gulp-sass` plugin for this to fully work. `npm i gulp-sass -D`).

```js
var gulp = require('gulp');
var fileLoader = require('gulp-file-loader');
var sass = require('gulp-sass');

// Preset SCSS gulp-file-loader task
gulp.task('sass:load', function(){
  // Always relative to gulpfile.js even if this code is inside a folder
  var dest = 'source/scss';
  // Do not leave off the "return", it is vital!
  return gulp.src('./source/components/**/*.scss')
    // Using the "scss" preset ("dest" must be provided here as well)
    .pipe(fileLoader({ preset: 'scss', dest: dest }))
    .pipe(gulp.dest(dest));
})

/************\
    Gulp 4
\************/

// Define a separate compile task
gulp.task('sass:compile', function(){
  return gulp
    .src('source/scss/main.scss')
    .pipe(sass())
    .pipe(gulp.dest('build/assets/css'));
})

// make "sass:load" run before "sass:compile" when "sass" is run
gulp.task('sass', gulp.series('sass:load', 'sass:compile'));

// Watch for changes
gulp.task('watch', function(done){
  gulp.watch('source/**/*.scss', gulp.series('sass'));
  done();
})

/************\
    Gulp 3
\************/

// Make "sass" dependent on "sass:load"
gulp.task('sass', ['sass:load'], function(){
  return gulp
    .src('source/scss/main.scss')
    .pipe(sass())
    .pipe(gulp.dest('build/assets/css'));
})

// Watch for changes
gulp.task('watch', function(){
  gulp.watch('source/**/*.scss', ['sass']);
})
```

The `scss` preset will apply the following default settings:

```js
// scss preset default settings
{
  format: '@import "$path";',
  fileName: 'file-loader.scss',
  retainOrder: true,
  header: `
// This file is generated by gulp-file-loader.
// Save this file into source control.
// You may rearrange the order of the imports however you like.
// You may NOT make any other alterations to this file.
`
};
```

The output of the example `'sass:load'` task will look something like this:

```scss
// This file is generated by gulp-file-loader.
// Save this file into source control.
// You may rearrange the order of the imports however you like.
// You may NOT make any other alterations to this file.

@import "../components/A/A.scss";
@import "../components/B/B.scss";
@import "../components/C/C.scss";
@import "../components/D/D.scss";
```

Due to the [`retainOrder: true` setting](#the-retainorder-setting), you can rearrange the output. Gulp File Loader will preserve the order when it recompiles.

```scss
// Rearrange the output and Gulp File Loader will preserve it
// (Requires the `retainOrder` setting to be enabled)

@import "../components/D/D.scss";
@import "../components/A/A.scss";
@import "../components/C/C.scss";
@import "../components/B/B.scss";
```

If you add a new file to the system (eg. a `../components/A/A-two.scss` file) gulp-file-loader will aim to group it with the other files found in the same folder.

```scss
// gulp-file-loader will aim to group new files with files found in the same folder

@import "../components/D/D.scss";
@import "../components/A/A.scss";
@import "../components/A/A-two.scss"; // New file added
@import "../components/C/C.scss";
@import "../components/B/B.scss";
```

Add this line to your main scss file to auto-loaded your component styles:

```scss
// Import the file-loader.scss file from main.scss
@import "./file-loader.scss";
```

You can now auto-load all of you're component scss files while still retaining full control over CSS source order! 😃


## Use in *combination* with [Gulp Sass Glob](https://www.npmjs.com/package/gulp-sass-glob)

If you are using Gulp File Loader to load scss files, it works best as a method used for loading your component files since those tend to require a specific order to work correctly.

Most of the time, your config files (files that hold nothing but Sass variables) and helper files (mixins, utility classes etc.) don't need to retain their order. It is both easier and cleaner to use [Gulp Sass Glob](https://www.npmjs.com/package/gulp-sass-glob) to auto-load these types of files than it is to set up individual Gulp File Loader tasks for each of them.

```
npm install gulp-sass-glob --save-dev
```

### Gulp 3 combination

```js
// Using Gulp File Loader in combination with Gulp Sass Glob in Gulp 3

var gulp = require('gulp');
var fileLoader = require('gulp-file-loader');
var sass = require('gulp-sass');
var sassGlob = require('gulp-sass-glob');

// Gulp File Loader task
gulp.task('sass:load', function(){
  var dest = 'source/scss';
  return gulp.src('./source/components/**/*.scss')
    .pipe(fileLoader({ preset: 'scss', dest: dest }))
    .pipe(gulp.dest(dest));
})

// Sass compile task (depends on 'sass:load' task)
gulp.task('sass', ['sass:load'], function(){
  return gulp
    .src('source/scss/main.scss')
    .pipe(sassGlob())// Sass Glob
    .pipe(sass())
    .pipe(gulp.dest('build/assets/css'));
})
```

### Gulp 4 combination

```js
// Using Gulp File Loader in combination with Gulp Sass Glob in Gulp 4

var gulp = require('gulp');
var fileLoader = require('gulp-file-loader');
var sass = require('gulp-sass');
var sassGlob = require('gulp-sass-glob');

// Gulp File Loader task
gulp.task('sass:load', function(){
  var dest = 'source/scss';
  return gulp.src('./source/components/**/*.scss')
    .pipe(fileLoader({ preset: 'scss', dest: dest }))
    .pipe(gulp.dest(dest));
})

// Sass compile task
gulp.task('sass:compile', function(){
  return gulp
    .src('source/scss/main.scss')
    .pipe(sassGlob())// Sass Glob
    .pipe(sass())
    .pipe(gulp.dest('build/assets/css'));
})

// Combined sass compile task
gulp.task('sass', gulp.series('sass:load', 'sass:compile'));
```

### Using a combination inside main.scss

```scss
/**********************\
    main.scss file
\**********************/

// Use Gulp Sass Glob to load config and helper files
@import "vars/**/*.scss";
@import "mixins/**/*.scss";

// Use the output from Gulp File Loader to load component files
@import "file-loader.scss";
```

```scss
/***************************\
    file-loader.scss file
  (automatically generated)
\***************************/

@import "../components/one/one.scss";
@import "../components/two/two.scss";
@import "../components/three/three.scss";
@import "../components/four/four.scss";
```

## All available presets

<dl>
  <dt><a href="https://github.com/Dan503/gulp-file-loader/blob/master/presets/es6.js">es6</a></dt>
  <dd>JavaScript imports that use the modern import syntax (<code>import thing from './file.js';</code>).</dd>

  <dt><a href="https://github.com/Dan503/gulp-file-loader/blob/master/presets/es5.js">es5</a></dt>
  <dd>JavaScript imports that use the old CommonJS syntax (<code>var thing = require('./file.js');</code>).</dd>

  <dt><a href="https://github.com/Dan503/gulp-file-loader/blob/master/presets/pug.js">pug</a></dt>
  <dd>Intended for use with builds that use <a href="https://pugjs.org/api/getting-started.html">Pug</a> as the templating language (<code>include ./file.pug</code>).</dd>

  <dt><a href="https://github.com/Dan503/gulp-file-loader/blob/master/presets/jade.js">jade</a></dt>
  <dd>For use on projects that haven't upgraded their old <a href="http://jade-lang.com/">Jade</a> powered projects to <a href="https://pugjs.org/api/getting-started.html">Pug</a> yet (<code>include ./file.jade</code>).</dd>

  <dt><a href="https://github.com/Dan503/gulp-file-loader/blob/master/presets/scss.js">scss</a></dt>
  <dd>Sass import statements that use the <a href="https://sass-lang.com/guide">newer SCSS style syntax</a> (<code>@import "./file.scss";</code>).</dd>

  <dt><a href="https://github.com/Dan503/gulp-file-loader/blob/master/presets/sass.js">sass</a></dt>
  <dd>Sass import statements that use the <a href="http://sass-lang.com/documentation/file.INDENTED_SYNTAX.html">older indented style syntax</a> (<code>@import ./file.sass</code>).</dd>

  <dt><a href="https://github.com/Dan503/gulp-file-loader/blob/master/presets/stylus.js">stylus</a></dt>
  <dd>Intended for use with the <a href="http://stylus-lang.com/">Stylus</a> CSS generation language (<code>@import "./file.styl"</code>).</dd>
</dl>

You can browse the available presets and see what their settings look like in [the `presets` folder](https://github.com/Dan503/gulp-file-loader/tree/master/presets). The presets are named after the file names in that folder.

If you would like other presets to be added, you can [log an issue](https://github.com/Dan503/gulp-file-loader/issues/new) to request for a new preset to be added, or you can make a pull request to add one yourself.

### Overriding a preset

The preset setting just tells Gulp File Loader what to use as the default settings.

You can override any of the preset settings by providing your own alternative setting. For example, to change the output file name, you can do this:

```js
// Overriding the default preset setting for "fileName"
.pipe(fileLoader({ preset: 'es6', fileName: 'different-file-name.js', dest: 'path/to/dest' }))
```

## JS configuration examples

Adding this functionality to your JS compiler can be tricky since JS compilers generally don't run off typical gulp functionality for performance reasons.

### Rollup

Rollup has a pretty straight forward integration. It is very similar to the Sass set up. It gets around the performance issues by allowing you to cache the last bundle that was generated.

Running `gulp start` will run the file loader, compile the JS, and then start watching files.

(Rollup by default does not bundle CommonJS `require()` statements).

#### Rollup in Gulp 4

```js
'use strict';

// Import file loader plugin
var fileLoader = require('gulp-file-loader');

var gulp = require('gulp');
var rollup = require('rollup-stream');
var sourcemaps = require('gulp-sourcemaps');
var source = require('vinyl-source-stream');
var buffer = require('vinyl-buffer');

gulp.task('js:load', function(){
  const dest = './source/js';
  return gulp.src([
    './source/components/**/*.js',
    // exclude files and folders starting with an underscore
    '!./source/components/{**/\_*,**/\_*/**}',
  ])
    // Run the file loader
    .pipe(fileLoader({ preset: 'es6', dest }))
    .pipe(gulp.dest(dest));
})

var cache;
gulp.task('js:compile', function() {
  return rollup({
      // point to the entry file.
      input: './source/js/main.js',
      sourcemap: true,
      // use cache for better performance
      cache: cache,
      // Intended for use with browsers
      format: 'iife',
    })
    .on('bundle', function(bundle) {
      // update cache data after every bundle is created
      cache = bundle;
    })
    // point to the entry file.
    .pipe(source('main.js', './source/js'))
    .pipe(buffer())
    .pipe(sourcemaps.init({loadMaps: true}))
    .pipe(sourcemaps.write('.'));

    .pipe(gulp.dest('./build/assets/js'));
});

gulp.task('js', gulp.series('js:load', 'js:compile'));

gulp.task('js:watch', function(done){
  gulp.watch(['./source/**/*.js'], gulp.series('js'));
  done();
})

gulp.task('start', gulp.series('js', 'js:watch'));
```

#### Rollup in Gulp 3

```js
'use strict';

// Import file loader plugin
var fileLoader = require('gulp-file-loader');

var gulp = require('gulp');
var rollup = require('rollup-stream');
var sourcemaps = require('gulp-sourcemaps');
var source = require('vinyl-source-stream');
var buffer = require('vinyl-buffer');

gulp.task('js:load', function(){
  const dest = './source/js';
  return gulp.src([
    './source/components/**/*.js',
    // exclude files and folders starting with an underscore
    '!./source/components/{**/\_*,**/\_*/**}',
  ])
    // Run the file loader
    .pipe(fileLoader({ preset: 'es6', dest }))
    .pipe(gulp.dest(dest));
})

var cache;
gulp.task('js', ['js:load'], function() {
  return rollup({
      // point to the entry file.
      input: './source/js/main.js',
      sourcemap: true,
      // use cache for better performance
      cache: cache,
      // Intended for use with browsers
      format: 'iife',
    })
    .on('bundle', function(bundle) {
      cache = bundle;
    })
    // point to the entry file.
    .pipe(source('main.js', './source/js'))
    .pipe(buffer())
    .pipe(sourcemaps.init({loadMaps: true}))
    .pipe(sourcemaps.write('.'))
    .pipe(gulp.dest('./build/assets/js'));
});

gulp.task('js:watch', function(){
  gulp.watch(['./source/**/*.js'], ['js']);
})

gulp.task('start', ['js', 'js:watch']);
```

### Browserify

Below is a modified version of the of the [Gulp browserify + watchify recipe](https://github.com/gulpjs/gulp/blob/master/docs/recipes/fast-browserify-builds-with-watchify.md) that has Gulp File Loader installed.

Browserify gets around performance issues by using a special "Watchify" JS library instead of `gulp.watch()`. You specify an entry file. Watchify watches for any changes to files that are imported from that entry file. It will also watch for changes to files imported from those imported files (and so on and so on forever).

The code below will generate a new `file-loader.js` file when it detects a JS file has been added to or removed from the components folder. The auto-generated `file-loader.js` file is imported into the main entry js file meaning Watchify is watching it for changes. When the new `file-loader.js` file is generated, Watchify detects that the file has changed. This triggers Watchify to initiate a Browserify rebundle.

Most of the code below works in both both Gulp 3 and Gulp 4.

```js
'use strict';

// Import file loader plugin
var fileLoader = require('gulp-file-loader');

var watchify = require('watchify');
var browserify = require('browserify');
var gulp = require('gulp');
var source = require('vinyl-source-stream');
var buffer = require('vinyl-buffer');
var log = require('gulplog');
var sourcemaps = require('gulp-sourcemaps');

var customOpts = {
  // entry file defined here
  entries: ['./source/js/main.js'],
  debug: true
};
var opts = Object.assign({}, watchify.args, customOpts);

// Watch for changes then bundle
var b = watchify(browserify(opts));

b.on('update', bundle); // on any dep update, runs the bundler
b.on('log', log.info); // output build logs to terminal

function bundle() {
  // Then bundle the code
  return b.bundle()
    .on('error', log.error.bind(log, 'Browserify Error'))
    .pipe(source('main.js'))
    .pipe(buffer())
    .pipe(sourcemaps.init({loadMaps: true}))
    .pipe(sourcemaps.write('./'))
    .pipe(gulp.dest('./build/assets/js'));
}

// File loader Gulp task
gulp.task('js:load', function(){
  const dest = './source/js';
  return gulp.src([
    './source/components/**/*.js',
    // exclude files and folders starting with an underscore
    '!./source/components/{**/\_*,**/\_*/**}',
  ])
    // Run the file loader
    .pipe(fileLoader({preset: 'es5', dest }))
    .pipe(gulp.dest(dest));
})


///////////////////////////
// Gulp 3 specific code //
/////////////////////////
gulp.task('js', ['js:load'], bundle); // so you can run `gulp js` to build the file
gulp.task('watch', function() {
  // Gulp 3 can't distinguish between 'add','unlink', and 'change' events
  // so it also has to run on file changes
  gulp.watch('./source/components/**/*.js', ['js:load']);
})


///////////////////////////
// Gulp 4 specific code //
/////////////////////////
gulp.task('js', gulp.series('js:load', bundle)); // so you can run `gulp js` to build the file
gulp.task('watch', function(done) {
  var watcher = gulp.watch('./source/components/**/*.js');
  //Gulp 4 has the advantage of only running when a file is added/removed, not changed
  watcher.on('add', gulp.series('js:load'));
  watcher.on('unlink', gulp.series('js:load'));
  done();
})
```

### Making use of the generated JS file

Now that Gulp is set up to build a file-loader JS file for you, import your generated file from main.js and call it as a function.

```js
// Import file-loader.js inside main.js
import fileLoader from './file-loader.js'; // ES6
var fileLoader = require('./file-loader.js'); // ES5

document.addEventListener("DOMContentLoaded", function() {
  // Run the file-loader code on page load
  fileLoader();
});
```

Note that a typical component js file will need to export a function by default for this configuration to work.

```js
// component js file example

  /////////
 // ES6 //
/////////
export default function on_page_load() {
  // Place code here that you wish to run
  // when the `fileLoader()` function is called
}

  /////////
 // ES5 //
/////////
module.exports = function on_page_load() {
  // Place code here that you wish to run
  // when the `fileLoader()` function is called
}
```

## How to do custom configurations

Now that you know how to use presets, lets replicate some of these presets manually to show you how to use the plugin if the files you wish to load are not available as a preset.

### Manual SCSS set up

I'll use SCSS as an example first because it is both simple and popular.

Create a gulp task that looks like this:

```js
// Typical SCSS gulp-file-loader task

var gulp = require('gulp');
var fileLoader = require('gulp-file-loader');

gulp.task('sass:load', function(){

  // Always relative to gulpfile.js even if this code is inside a folder
  var dest = 'source/scss';

  // Do not leave off the "return", it is vital!
  return gulp.src([
    // These paths are always relative to gulpfile.js
    './source/components/**/*.scss',
    // Ignore files & folders that start with underscores
    '!./source/{**/\_*,**/\_*/**}',
  ])
    .pipe(fileLoader({
      // "$path" is replaced with a relative file path
      format: '@import "$path";',
      // destination folder (must match gulp.dest)
      dest: dest,
      // name of the output file
      fileName: 'file-loader.scss',
      // Don't change the order that imports are currently in
      retainOrder: true,
      // Add a message to the top of the file
      header: "// output from gulp-file-loader",
    }))
    .pipe(gulp.dest(dest));
})
```

The output of this Gulp task will look something like this:

```scss
// output from gulp-file-loader
@import "../components/A/A.scss";
@import "../components/B/B.scss";
@import "../components/C/C.scss";
@import "../components/D/D.scss";
```

### Manual JS set up

JS is slightly more complicated.

```js
// Typical JS gulp-file-loader task

var gulp = require('gulp');
var fileLoader = require('gulp-file-loader');

// Use an ES6 template literal for defining the template
// Node has supported them natively ever since v4.0.0
var template = `
$format[imports]

export default function(){
$format[functions]
}
`;

gulp.task('js:load', function(){

  var dest = 'source/js';

  return gulp.src([
    './source/components/**/*.js',
    // Ignore files & folders that start with underscores
    '!./source/{**/\_*,**/\_*/**}'
  ])
    .pipe(fileLoader({
      // Format is now split into an object holding named format strings
      format: {
        // "$name" is replaced with the name of the file
        // "$path" is replaced with a relative path to the file
        imports: 'import $name from "$path";',
        // The indent is added here, not in the template
        functions: '  $name();'
      },
      dest: dest,
      fileName: 'file-loader.js',
      template: template
    }))
    .pipe(gulp.dest(dest));

})
```

The output from this task will look something like this:

```js
// Generated file-loader.js file

import one from "../components/one/one.js";
import two from "../components/two/two.js";
import three from "../components/three/three.js";
import four from "../components/four/four.js";

export default function(){
  one();
  two();
  three();
  four();
}
```

## Understanding the `format` and `template` settings

It is recommended that you use an ES6 Template Literal (the back tick style strings) for creating the template rather than regular strings. Template Literals will allow you to define the markup found inside the output file exactly as written in a single string. Regular strings don't accept new lines so it makes writing the template much more difficult.

Remember that Template Literal's count all white space literally, so any white space you add to the template will appear in the final output. To avoid an odd looking output file, save the template to a `template` variable outside of the gulp task so that there is no indentation to the side of it.

The template works by replacing each `$format[formatName]` statement with a full list of imports formated in the specified way provided in the `format` object.

If the `format` setting is provided as a string, the `template` setting is ignored. If `format` is provided as an object, the `template` setting is required.

### The `$name` placeholder

The `$name` placeholder in the `format` setting is replaced with the file name of the file. Any non-alphabetic and non-numeric characters are converted to underscores to prevent possible syntax errors.

```
./folder/path/gulp-file-loader-is-awesome!123.js

 === converts to the $name ===

gulp_file_loader_is_awesome_123
```

If there are duplicate file names, a number is added to the end of the name based on how many duplicates it has found to ensure that each name is unique.

```
./folder/one/thing.js
./folder/two/thing.js

 === converts to the $name ===

thing
thing_1
```

The `$name` placeholder is excellent for use cases where you need to assign an import path to a variable name.

You can use the `$name` placeholder as much as you like. That includes having the `$name` placeholder appear multiple times in a single format rule. The `$name` will always refer to the same import path.

### The `$path` placeholder

The `$path` placeholder in the `format` setting is replaced with a relative path that goes from the file-loader output file to the file that is being loaded in.

```
$path = ./path/to/file.ext
```

`$path` can only be declared once per format rule.

### Using indents

If you want indenting, the **indenting should be added through the `format` setting** _not_ the `template` setting. If you indent the template, only the first item in the list will be indented. The rest will press hard up against the edge of the page.

For example, if you use this as your template:

```js
// How NOT to indent your template

var template = `
$format[imports]

export default function(){
    // Notice the indent here
    $format[functions]
}
`;

```

You will end up with a JS file that looks like this:

```js
// The result of incorrect indentation

import one from "../components/one/one.js";
import two from "../components/two/two.js";
import three from "../components/three/three.js";
import four from "../components/four/four.js";

export default function(){
    // Notice the indent here
    one();
two();
three();
four();
}
```

Instead, apply indentation through the `format` setting:

```js
//How to apply correct indentation

var template = `
$format[imports]

export default function(){
    // Notice the indent here
$format[functions]
}
`;

// ... other Gulp code ...

.pipe(fileLoader({
  format: {
    imports: 'import $name from "$path";',
    // The indent is added here, not in the template
    functions: '    $name();'
  },
  dest: dest,
  fileName: 'file-loader.js',
  template: template
}))
```

That will produce the desired output:

```js
// The result of correct indenting

import one from "../components/one/one.js";
import two from "../components/two/two.js";
import three from "../components/three/three.js";
import four from "../components/four/four.js";

export default function(){
    // Notice the indent here
    one();
    two();
    three();
    four();
}
```

## The `retainOrder` setting

I briefly touched on the `retainOrder` setting earlier, however there is a bit more to know about it.

In CSS, the order that styles are written in matters significantly. It is important that you are able to alter the order that files are loaded in if you wish to have full control over your CSS specificity.

Other globing methods (eg. `@import "../path/to/components/**/*.scss";`) do not give you the ability to alter the order that the files are loaded in. You are generally restricted to loading files in alphabetical order. Gulp File Loader gives you back the ability to control the order that your CSS loads in with it's `retainOrder` setting (introduced in v2.0.0).

By default `retainOrder` is set to `false`. When `retainOrder` is set to `true`, Gulp File Loader will not alter the order of the existing import paths if you manually edit them yourself. Make sure that if you enable the `retainOrder` setting you **save the output file into source control**. This will ensure that your co-workers don't end up with a CSS file that is in a different order to yours.

Gulp File Loader will still delete old files from the list that don't exist any more.

If it detects that a new file is added to the system, Gulp File Loader will aim to keep that new file grouped with other files found in the same folder. (Prior to v2.1.0 it just dumped it at the bottom of the file). This means that new scss config file imports will be placed at the top of the file-loader file with the other config files. This gives all your component files access to the new config settings without you having to make any alterations to the imports file.

It will not retain any comments or other alterations to the file. It will only retain the order that the imports were announced in.

## Settings reference guide

<dl>
  <dt>header</dt>
  <dd>A string of text that is added to the <strong>top</strong> of the output file when it is generated. This is good for leaving comments like "<code style="white-space: nowrap;">// Do not edit this file</code>".</dd>

  <dt>footer</dt>
  <dd>A string of text that is added to the <strong>bottom</strong> of the output file when it is generated. This might be useful for calling a custom function at the bottom of the file after all the imports have been loaded.</dd>

  <dt>format</dt>
  <dd>The format setting dictates the format of each import line in the generated file. It can either be a string or an object holding multiple named strings. If you provide an object to this setting, the <code>template</code> setting is also required. Use the <code>$path</code> and <code>$name</code> placeholders inside the string to determine where the file path and name should go.</dd>

  <dt>template</dt>
  <dd>The template setting holds a string that dictates the overall structure of the generated file. Use <code>$format[formatName]</code> placeholders in the template string to dictate where each format should be used in the output file. If the <code>format</code> setting is provided as a string, the <code>template</code> setting is ignored.</dd>

  <dt>fileName</dt>
  <dd>A string holding the file name for the output file. The file extension must be included.</dd>

  <dt>dest</dt>
  <dd>Should be identical to the setting used in <code>gulp.dest</code>. It determines where the output file is sent after processing. It is <strong>always</strong> a relative path from <code>gulpfile.js</code>,  <strong>not a relative path from the current file</strong>. Example: <code>"path/to/destination"</code>. The <code>gulp.dest</code> setting is unavailable at the time the plugin is called, thus the setting needs to be provided explicitly.</dd>

  <dt>retainOrder</dt>
  <dd>This is set to <code>false</code> by default. When set to <code>true</code>, it will never alter the order that imports are loaded in. This gives you the ability to manually edit the output file to achieve the desired import order. Make sure to <strong>save the output file into source control</strong> so that your team mates end up with a file that is in the same order as yours.</dd>

  <dt>preset</dt>
  <dd>Use a set of predefined default settings rather than configuring all of the settings yourself. You can view the available presets and the settings that they provide in <a href="https://github.com/Dan503/gulp-file-loader/tree/master/presets">the <code>presets</code> folder</a>.</dd>
</dl>

## Change Log

The Change log can be viewed on the [Gulp File Loader GitHub releases](https://github.com/Dan503/gulp-file-loader/releases) page.
